//**********************************************************************
// kill
// ====
// Kill a process
//**********************************************************************

#ifndef STRICT
#define STRICT
#endif

#include <windows.h>
#include <stdio.h>


//**********************************************************************
// Prototypes
// ----------
//**********************************************************************

BOOL strtoint(const char* s, int* i);
const char* GetLastErrorMessage(void);


//**********************************************************************
// Global variables
// ----------------
//**********************************************************************

#define SYNTAX \
"Syntax: kill <process ID>\n\n" \
"Kill a process.  This will kill any process if you have sufficient privileges\n" \
"so use with care!\n"


//**********************************************************************
// main
// ----
//**********************************************************************

int main(int argc, char* argv[])
{ int pid;
  HANDLE token, hprocess;
  TOKEN_PRIVILEGES tkp;

// Check the arguments

  if (argc != 2)
  { printf(SYNTAX);
    return(2);
  }

  if (strcmp(argv[1], "-?") == 0 || strcmp(argv[1], "/?") == 0)
  { printf(SYNTAX);
    return(0);
  }

// Get the process id

  if (!strtoint(argv[1], &pid))
  { printf("\"%s\" is not a valid integer\n", argv[1]);
    return(2);
  }

// Grant ourselves debug privilege

  if (OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &token))
  { LookupPrivilegeValue(NULL, SE_DEBUG_NAME, &tkp.Privileges[0].Luid);
    tkp.PrivilegeCount = 1;
    tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    AdjustTokenPrivileges(token, FALSE, &tkp, 0, NULL, 0);
  }

// Open the process

  hprocess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, (DWORD) pid);
  if (!hprocess)
  { printf("Cannot open process %i:\n%s\n", pid, GetLastErrorMessage());
    return(2);
  }

// And kill it

  if (!TerminateProcess(hprocess, 0))
  { printf("Cannot terminate process:\n%s\n", GetLastErrorMessage());
    return(2);
  }

// All done

  return(0);
}


//**********************************************************************
// strtoint
// --------
// Convert string to int
//**********************************************************************

BOOL strtoint(const char* s, int* i)
{ const char* start;
  char* end;

  for (start = s; *start == ' ' || *start == '\t'; start++);
  *i = (int) strtol(start, &end, 10);

  return(end > start ? TRUE : FALSE);
}


//**********************************************************************
// GetLastErrorMessage
// -------------------
//**********************************************************************

const char* GetLastErrorMessage(void)
{ DWORD d;
  static char errmsg[1024];

  d = GetLastError();

  strcpy(errmsg, "<unknown error>");
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, d, 0, errmsg, 1023, NULL);
  return(errmsg);
}


